module Roqua
  module RomApi
    module Sessions
      class BasicAuthSession
        attr_reader :rom_host
        attr_reader :username
        attr_reader :password

        def initialize(rom_host: ENV['ROM_SITE'],
                       username: ENV['ROM_BASICAUTH_ID'],
                       password: ENV['ROM_BASICAUTH_SECRET'])
          @rom_host = rom_host
          @username = username
          @password = password
        end

        def get(path, params = {})
          perform_request_or_fail do
            HTTParty.get(full_url_for(path),
                         query: params,
                         basic_auth: basic_auth)
          end
        end

        def post(path, params = {})
          perform_request_or_fail do
            HTTParty.post(full_url_for(path),
                          headers: {'Content-Type' => 'application/json'},
                          body: params.to_json,
                          basic_auth: basic_auth)
          end
        end

        def patch(path, params = {})
          perform_request_or_fail do
            HTTParty.patch(full_url_for(path),
                           headers: {'Content-Type' => 'application/json'},
                           body: params.to_json,
                           basic_auth: basic_auth)
          end
        end

        def delete(path, params = {})
          perform_request_or_fail do
            HTTParty.delete(full_url_for(path),
                            query: params,
                            basic_auth: basic_auth)
          end
        end

        private

        def perform_request_or_fail(&block)
          response = yield
          case response.code
          when 200..299, 422
            response
          when 401
            access_denied(response)
          else
            msg = response.parsed_response
            if msg.is_a? Hash
              msg = msg.fetch('error', nil)
            end
            fail msg || "Received HTTP response code #{response.code}!"
          end
        end

        def access_denied(response)
          if response.headers['WWW-Authenticate']
            fail NoSession
          else
            fail Unauthorized
          end
        end

        def full_url_for(path)
          rom_host + api_base + path + '.json'
        end

        def api_base
          '/api/v1'
        end

        def basic_auth
          {username: username, password: password}
        end
      end
    end
  end
end
